<?php
///////////////////////////////////////////////////////////////////////////

class Newrus_Api
{
	const API_URL = 'http://api.new-rus.tv/api/json/';
	private $cookie = null;
	public $settings = null;
	public $account = null;
	private $is_logged = null;
	private $disk_space = 0;


	public function is_logged_in()
	{ return $this->is_logged; }

	public function get_account()
	{ return $this->account; }

	public function get_settings()
	{ return $this->settings; }

///////////////////////////////////////////////////////////////////////////

    	public function api_exception($e, $def_caption, $is_playback)
	{
		$is_newrus_error = ($e instanceof Newrus_Exception);
        	if (!$is_newrus_error)
        	{
            		hd_print('General exception: ' . $e->getMessage());
            		$title = !$is_playback ? 'Error' :
                	'Не удалось подключиться к Интернету.';
            		$text_lines = $is_playback ? array() :
                	array(
                 		'Не удалось подключиться к Интернету.',
                 		'Пожалуйста, проверьте подключение к Интернету.'
                		);
            		return new DuneException(
                	$def_caption, -1,
                		ActionFactory::show_error(false, $title, $text_lines));
        	}
        	hd_print('Newrus API exception: ' .
            		'code=' . $e->getCode() . ', message=' . $e->getMessage());

        	$ecode = $e->getCode();
		$emessage = $e->getMessage();
        	#$fatal = $ecode == 5 || $ecode == 6 ||
            		#$ecode == 11 || $ecode == 12 || $ecode == 13;

		$fatal = $ecode == 12 && $emessage == 'Authentification error' || $ecode == 17 || $ecode == 5;
        	if ($fatal)
            		$this->unset_login();

		#if ($ecode == 2 || $ecode == 4)
		#	{
		#		$this->is_logged = false;
		#	}
        	$title = !$is_playback ? 'Error' : $emessage;
        	$text_lines = $is_playback ? array() :
            	array('Описание: ' . $emessage,
                	'Код ошибки: ' . $ecode);
        	return new DuneException(
            		$def_caption, $ecode,
            		ActionFactory::show_error($fatal, $title, $text_lines));
		
	}
///////////////////////////////////////////////////////////////////////////
	public function unset_login()
	{
		hd_print("unset_login");
		$this->cookie = null;
		$this->is_logged = false;
		$this->account = null;
		$this->settings = null;
		
	}
///////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
	public function check_login($plugin_cookies)
	{
		hd_print("check_login");
		hd_print("cookie:$this->cookie");
		$is_login_correct = isset($plugin_cookies->is_login_correct) ? $plugin_cookies->is_login_correct : false;
		hd_print("is_login_corect:$is_login_correct");
		$is_logged = $this->is_logged_in();
		hd_print("is_logged:$is_logged");
		$is_log = isset($is_logged) ? $is_logged : false;
		if ($is_login_correct == 1 && $is_logged != 1)
		{
			hd_print("Авторизация>");
			$this->login_newrus($plugin_cookies);
			hd_print("cookie1:$this->cookie");
		}	
	}

///////////////////////////////////////////////////////////////////////////
	public function logout()
	{
		$this->unset_login();
	}
///////////////////////////////////////////////////////////////////////////
	public function login_newrus($plugin_cookies)
	{
		if (!isset($plugin_cookies->login))
            		throw new Exception('Логин не введен');
		if (!isset($plugin_cookies->pass))
            		throw new Exception('Пароль не введен');
		$doc = self::API_URL . 'login.php?login='. $plugin_cookies->login.'&pass='.$plugin_cookies->pass.'&settings=all';
		hd_print("--->>> get_login: $doc");
		try
		{
			$this->is_logged = true;
			$plugin_cookies->is_login_correct = true;
			$data = $this->json_fetch($doc,1);
		}
		catch (Exception $e)
		{
			$this->is_logged = false;
			hd_print("loged falled");
			$plugin_cookies->is_login_correct = false;
			throw $this->api_exception($e,'Ошибка авторизации', false);
		}
		if(isset($data->sid) && isset($data->sid_name))
		{
			$this->cookie = strval($data->sid_name).'='.strval($data->sid);
			$this->settings = $data->settings;
			$this->account = $data->account;
			$this->test_free_space();
			try
			{
				hd_print("bitrate");
				$this->set_settings('bitrate',4000);
			}
			catch (Exception $e)
			{
				throw $this->api_exception($e,'Ошибка смены битрейта', false);
			}
			

		}
		
	}
///////////////////////////////////////////////////////////////////////////
	public function test_free_space()
	{
		$plugin_dir = DuneSystem::$properties['install_dir_path'];
		$free_space = disk_free_space("$plugin_dir");
		$url_prefix = Newrus_Config::ARCHIVE_URL;
		$size_url = $url_prefix . '/size.txt';
		$doc_ = HD::http_get_document($size_url);
		if (is_null($doc_))
            	{
                	hd_print("Failed to fetch archive size.txt from $size_url.");
			$this->disk_space = 0;
            	}
            	else
            	{
                	$total_size = intval($doc_);
                	hd_print(" size = $total_size");
			if($free_space >= $total_size)
			{
				$this->disk_space = 1;
			}
			else
			{
				hd_print("low disk space");
				$this->disk_space = 0;
			}
            	}
	}
///////////////////////////////////////////////////////////////////////////
	public function get_channel_list()
	{
		$doc = self::API_URL.'channel_list.php?have_sepg=0';
		hd_print("--->>> get_channel_list: $doc");
		try
		{
			$data = $this->json_fetch($doc,0);
		}
		catch (Exception $e)
		{
			throw $this->api_exception($e,'get_channel_list failed.', true);
		}
		return $data;	
	}
///////////////////////////////////////////////////////////////////////////
	public function get_epg($cid,$day)
	{
		$doc = self::API_URL.'epg.php?cid='.$cid.'&day='.$day;
		hd_print("--->>> get_epg: $doc");
		try
		{
			$data = $this->json_fetch($doc,0);
		}
		catch (Exception $e)
		{
			throw $this->api_exception($e,'get_epg failed.', true);
		}				
		return $data;	
	}
///////////////////////////////////////////////////////////////////////////
	public function get_url($cid,$gmt,$protect_code)
	{
		$doc = self::API_URL.'get_url.php?cid='.$cid;
		hd_print("--->>> get_url: $doc");
		if($gmt >= 0)
		{
			$doc .= "&gmt=$gmt";
		}
		if(isset($protect_code))
		{
			$doc .= "&protect_code=$protect_code";
		}
		try
		{
			$data = $this->json_fetch($doc,0);
		}
		catch (Exception $e)
		{
			throw $this->api_exception($e,'get_url failed.', true);
		}				
		$url_ = $data->url;
		hd_print("url do $url_");
		if ( $gmt >= 0 )
		{
			$url_ = explode(" :", $url_);
			$url = str_ireplace("http/ts://","http://",$url_[0]);
		}
		else
		{
			
			#$url = str_ireplace("http://","http://ts://",$url_);
			$url = $url_;
		}
		$urll = DuneSystem::$properties['plugin_cgi_url'];
		$urll = str_ireplace("http://","http://ts://",$urll);
		$urll .= "nrs.sh?$url";
		hd_print("url posle $urll");
		return $urll;	
	}

///////////////////////////////////////////////////////////////////////////

	public function get_vod_genres()
	{
		$doc = self::API_URL.'vod_genres.php';
		hd_print("--->>> get_vod_genres: $doc");
		try
		{
			$data = $this->json_fetch($doc,0);
		}
		catch (Exception $e)
		{
			throw $this->api_exception($e,'get_vod_genres failed.', false);
		}				
		return $data;	
	}

///////////////////////////////////////////////////////////////////////////
//TODO 2
	public function get_vod_list($aid,$bid)
	{
		if($aid == 'search')
		{
			$bid = urlencode($bid);
			$doc = self::API_URL.'vod_list.php?query='.$bid.'&genre=0';
		}
		else
		{
			$doc = self::API_URL.'vod_list.php?genre='.$bid;
		}

		hd_print("--->>> get_vod_list: $doc");
		try
		{
			$data = $this->json_fetch($doc,0);
		}
		catch (Exception $e)
		{
			throw $this->api_exception($e,'get_vod_list failed.', false);
		}				
		return $data;	
	}

///////////////////////////////////////////////////////////////////////////
	public function get_vod_info($id)
	{
		$doc = self::API_URL.'vod_info.php?id='.$id;
		hd_print("--->>> get_vod_info: $doc");
		try
		{
			$data = $this->json_fetch($doc,0);
		}
		catch (Exception $e)
		{
			throw $this->api_exception($e,'get_vod_list failed.', false);
		}				
		return $data;	
			
	}

///////////////////////////////////////////////////////////////////////////
	public function get_vod_url($fileid)
	{
		$doc = self::API_URL.'vod_geturl.php?fileid='.$fileid;
		hd_print("--->>> get_vod_url: $doc");
		try
		{
			$data = $this->json_fetch($doc,0);
		}
		catch (Exception $e)
		{
			throw $this->api_exception($e,'get_vod_list failed.', false);
		}				
		return $data;	
	}

///////////////////////////////////////////////////////////////////////////
//TODO 3
	public function set_settings($control_id, $new_value)
	{
		if($control_id == 'pcode')
		{
			$doc = self::API_URL.'settings_set.php?var='.$control_id.$new_value;
		}
		else
		{
			$doc = self::API_URL.'settings_set.php?var='.$control_id.'&val='.$new_value;
		}
		hd_print("--->>> set_settings: $doc");
		try
		{
			$data = $this->json_fetch($doc,0);
		}
		catch (Exception $e)
		{
			throw $this->api_exception($e,"get settings $control_id failed.", false);
		}
		

		if (intval($data->message->code) == 4)
		{	if($control_id != 'pcode')			
			 $this->settings->{$control_id}->value = $new_value;
		}
		else
		{
			throw new Newrus_Exception($data->error->message, $code,false);
		}
	
	}

///////////////////////////////////////////////////////////////////////////
//TODO 4
	public function set_channels_order($id,$act)
	{
		if($id == -1)
		{
			$doc = self::API_URL.'set_channels_order.php?act='.$act;
		}
		else
		{
			$doc = self::API_URL.'set_channels_order.php?id='.$id.'&act='.$act;
		}
		hd_print("--->>> set_channels_order: $doc");
		$data = $this->json_fetch($doc,0);
		if (intval($data->message->code) == 4)
		{	
			return $data;
		}
		else
		{
			throw new Newrus_Exception($data->error->message, $code);
		}

	}

///////////////////////////////////////////////////////////////////////////
	public function get_channels_order()
	{
		$doc = self::API_URL.'get_channels_oder.php';
		hd_print("--->>> get_channels_order: $doc");
		$data = $this->json_fetch($doc,0);
		return $data;	
	}

///////////////////////////////////////////////////////////////////////////
	public function json_fetch($url,$islogin)
	{
		if($islogin == 0)
		{
			if (strpos($url,"?") === false )
			{
		    		$url .= "?$this->cookie";
			}
			else
			{
		    		$url .= "&$this->cookie";
			}
		}
		$doc = HD::http_get_document($url);
		#hd_print("--->>> get_json: $doc");
		$data = json_decode($doc);
		if ($data === null)
                throw new Exception('Invalid data received from server');
		if (isset($data->error))
            	{
			$code = intval($data->error->code);
			hd_print("API error: URL '$url' " .
                    	"returned error with code=$code, msg=" .
                    	$data->error->message . '.');
			throw new Newrus_Exception($data->error->message, $code);
		}
		return $data;
	}
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
    public function do_get_edit_subscription_defs(&$plugin_cookies, $handler, $add_params)
	{
		$defs = array();
		
		$login = isset($plugin_cookies->login) ? $plugin_cookies->login : '';
        	$pass = isset($plugin_cookies->pass) ? $plugin_cookies->pass : '';
		ControlFactory::add_text_field($defs,
            		$handler, $add_params,
            			'login', 'Подписка:',
            			$login, true, false, false, true, 500);
        	ControlFactory::add_text_field($defs,
            		$handler, $add_params,
            		'pass', 'Пароль:',
            		$pass, true, true, false, false, 500);

        	ControlFactory::add_vgap($defs, 50);

        	ControlFactory::add_button($defs,
            		$handler, $add_params,
            		'apply_sub', null, 'Применить', 300);

        	ControlFactory::add_vgap($defs, -3);

		ControlFactory::add_close_dialog_button($defs,
            		'Отменить', 300);

        	#ControlFactory::add_button($defs,$handler,$add_params,
            	#				'close',null,'Отменить', 300);

        	return $defs;
	}
/////////////////////////////////////////////////////////////////////////// 
   public function do_get_edit_subscription_action(&$plugin_cookies, $handler, $add_params = null) 
	{
        	return ActionFactory::show_dialog(
            		'Введите данные логин и пароль',
            		$this->do_get_edit_subscription_defs(
                	$plugin_cookies, $handler, $add_params),
            		true);

	} 
///////////////////////////////////////////////////////////////////////////
    public function apply_sub(&$plugin_cookies, &$user_input)
    {
        if (!isset($user_input->control_id) ||
            $user_input->control_id != 'apply_sub')
        {
	    #hd_print("dalshe: $user_input->control_id");
            return false;
	}
	if ($user_input->control_id == 'close')
        {
		hd_print("close");
		return array(
               'need_close_dialog' => true,
                'action' =>ActionFactory::close_dialog());
        }


        if ($user_input->login === '' || $user_input->pass === '')
        {
		return array(
                'need_close_dialog' => false,
                'action' =>ActionFactory::show_error(false,'Error',array('Поле Подписка не может быть пустым.')));
        }

	$plugin_cookies->login = $user_input->login;
	$plugin_cookies->pass = $user_input->pass;
	$islogged = $this->is_logged_in();
	if ($islogged == true)
	{
        	$this->logout();
	}
        $post_action = null;
        try
        {
            $this->login_newrus($plugin_cookies);
        }
        catch (DuneException $e)
        {
            $post_action = $e->get_error_action();
        }
	if ($plugin_cookies->is_login_correct === 0)
	{
		return array(
                'need_close_dialog' => false,
                'action' =>ActionFactory::show_error(false,'Error',array('Неправильный логин или пароль')));
	}
        return array(
            'need_close_dialog' => true,
            'action' => $post_action);
    }
///////////////////////////////////////////////////////////////////////////
    public function get_archive()
    {
	if ($this->disk_space == 1)
	{
        	return DefaultArchive::get_archive('ico', Newrus_Config::ARCHIVE_URL);
	}
	else
	{
		return null;
	}
    }

    public function get_icon($icon)
    {
        #hd_print("disk_space:$this->disk_space");
	if ($this->disk_space == 1)
	{
        	$archive = DefaultArchive::get_archive('ico',Newrus_Config::ARCHIVE_URL); 
        	return $archive->get_archive_url($icon);
	}
	else
	{
		$icon_url = "http://api.new-rus.tv/_logos/duneLogos/CACHE_NEWRUS/".$icon;
		#hd_print("eee:$icon_url");
		return $icon_url;
	} 
 
    }	


}
